<?php
/* Copyright 2007, 2008 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
/** Defines import/export capabilities for the library.
 * The calling controller should have access to all models (i.e. have them
 * in its $uses variable).
 */
class ImportExportComponent extends Object {
	public $components = array('File', 'CSVImporter');
	private $controller;
	private $errors;
	
	public function startup(&$controller) {
		$this->controller =& $controller;
		$this->errors = array();		
	}
	
	/** Logs an error message.
	 *
	 * @param string $message The message to log. Will be localized.
	 * @return boolean false (always)
	 */
	private function error($message) {
		$this->errors[] = __($message, true);
		return false;
	}
	
	/** Returns a log of errors that occured during processing.
	 *
	 * @return string
	 */
	public function getErrors() {
		return $this->errors;
	}
	
	/** Exports the database.
	 *
	 * @param string $method The export type.
	 * @param boolean $withResources true if associated resources (most notably
	 * images) should be packed with the data.
	 * @return An array with mimetype and content or zip file info, depending
	 * on wether the resources have been included or not. False in case of
	 * error, see {@link #getErrors} for the log.
	 */
	public function export($method, $withResources = false) {
		// Just in case we would need it, act before $method is modified
		$retZipName = "demyo-export-$method.zip";
		
		$exported = array();
		$options = array();
		
		// Detect method
		switch ($method) {
			case 'xml+xsl':
				$options['XSL'] = true;
				// Fall-through
			case 'xml':
				$exported['mimetype'] = 'application/xml';
				$method = 'to_xml';
				break;
			default:
				return $this->error('Invalid export method');
		}
		
		// Run export
		if (! $ret = $this->$method($options))
			return $this->error('Export method failed');
		list($filename, $content) = $ret;
		
		// Return data, or build a zip with the resources
		if (! $withResources)
			return array_merge($exported, array('content' => $content));
		
		// Reset the mimetype: we'll be sending a zip no matter what
		$exported['mimetype'] = 'application/zip';

		$zipName = tempnam(TMP, 'demyo-export-');
		unlink($zipName); // ZipArchive needs a .zip 
		$zip = $zip = new ZipArchive();
		$zipName .= '.zip';
		if ($zip->open($zipName, ZipArchive::CREATE) !== true)
			return $this->error(__('Failed to create ZIP archive.'));;
		
		$zip->addFromString($filename, $content);
		$this->File->addDirToZip($zip, DEMYO_IMAGES);
		
		$zip->close();
		//register_shutdown_function('unlink', $zipName); // Delete on exit, done from the view (meh, Cake doesn't look adapted for readfile transfers)
		
		return array_merge($exported, array('zipFileName' => $retZipName, 'zipPath' => $zipName));
	}
	
	public function import($method, $formFileData) {
		switch ($method) {
			case 'xml':
				$method = 'from_xml';
				break;
			case 'csv':
				$method = 'from_csv';
				break;
			default:
				return $this->error('Invalid import method');
		}

		if ($formFileData['error'] !== UPLOAD_ERR_OK)
			return $this->error('Upload failed with <a href="http://php.net/manual/en/features.file-upload.errors.php">error code '.$formFileData['error'].'</a>');
		$ext = pathinfo($formFileData['name'], PATHINFO_EXTENSION);
		
		$parseable = $formFileData['tmp_name'];
		if ($ext == 'zip') { // Unzip
			$tmpFolder = TMP.DS.'zip_tmp';
			if (file_exists($tmpFolder) && !$this->File->rm($tmpFolder))
				return $this->error('Failed to remove old archive directory');
			if (! mkdir($tmpFolder, 0700))
				return $this->error('Failed to make archive directory');
			
			$zip = new ZipArchive();
			if ($zip->open($formFileData['tmp_name']) !== true)
				return $this->error('Failed to open zip archive');
			if (! $zip->extractTo($tmpFolder))
				$this->error('Failed to extract archive');
			$zip->close();
			
			// Delete all files but root
			if (! $this->File->rm(DEMYO_IMAGES, true))
				return $this->error('Failed to delete old images');
			// Move contents from temporary extraction to final directory
			if (! $this->File->moveDirContent($tmpFolder.DS.DEMYO_IMAGES_DIR, DEMYO_IMAGES))
				return $this->error('Failed to restore images');
			// Give rights in final directory. See bug #91
			if (! $this->File->chmod(DEMYO_IMAGES, 0666))
				return $this->error('Failed to set rights for images');
			
			switch ($method) {
				case 'from_xml':
					$parseable = $tmpFolder.DS.'demyo.xml';
					break;
				default:
					return $this->error('No parseable file for archive');
			}
		}
		
		return $this->$method($parseable);
	}
	
	private function to_xml($options) {
		$withXSL = isset($options['XSL']) && $options['XSL'];
		
		// Prepare header before trying to export (fast-fail)
		$ret = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
		$style = '';
		if ($withXSL) {
			$ret .= <<<XSL
<?xml-stylesheet href="#style" type="text/xsl"?>
<!DOCTYPE doc [
<!ATTLIST xsl:stylesheet id ID #REQUIRED>
]>

XSL;
			$style = file_get_contents(WWW_ROOT.DS.'files'.DS.'demyo.xsl');
			if ($style === false)
				return $this->error('Failed to open stylesheet.');
		}
		
		
		// Build XML
		$output = new SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><library version="1.0">'.$style.'</library>');
		foreach (array('Publisher', 'Collection', 'Binding', 'Author', 'Tag',
			'Image', 'Series', 'Album', 'DerivativeType', 'Source', 'Derivative',
			'Borrower', 'AlbumsBorrower') as $model) {
		
			$this->controller->$model->addToElement($output);
		}
		
		// Format XML
		$doc = new DOMDocument('1.0', 'utf-8');
		$doc->formatOutput = true;
		$domnode = dom_import_simplexml($output);
		$domnode = $doc->importNode($domnode, true);
		$domnode = $doc->appendChild($domnode);
		
		return array('demyo.xml', $ret.$doc->saveXML($domnode));
	}
	
	private function from_xml($parseable) {
		if (($xml = simplexml_load_file($parseable)) === false)
			return $this->error('Failed to load XML');
		$xml = current($xml->xpath('//library'));
		
		// When importing, the order does matter: the database potentially
		// checks references 
		foreach (array('Publisher', 'Collection', 'Binding', 'Author', 'Tag',
				'Image', 'Series', 'Album', 'DerivativeType', 'Source',
				'Derivative', 'Borrower', 'AlbumsBorrower')
				as $modelName)
			if (!$this->controller->$modelName->importXML($xml))
				return false;
		
		return true;
	}
	
	private function from_csv($parseable) {
		$warnings = $this->CSVImporter->import($this->controller, $parseable);
		if (count($warnings) <= 0)
			return true;
		$this->errors = array_merge($this->errors, $warnings);
		return false;
	}
}
?>